==> caiviar-0.1.18.tar.gz: caiviar-0.1.18/server/ServerSocket.cpp <==
/* ServerSocket.cpp

java-like implementation of sockets.

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Peter Dikant <peter.dikant@mobilexag.de>
Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#if defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>
#include <memory.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <net/if.h>
#include <netdb.h>
#elif defined(WIN32)
#ifndef __windows_h_included__
#define __windows_h_included__
#include <windows.h>
#endif
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#endif
#include "ServerSocket.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#if defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
static void Sleep(int useconds)
{
	usleep(useconds);
}
static void closesocket(int socket)
{
	close(socket);
}
static const char FALSE = 0;
static const char TRUE = 1;
typedef sockaddr_in SOCKADDR_IN;
typedef sockaddr_in* LPSOCKADDR;
#endif

ServerSocket::ServerSocket() 
{

}

/**
 * close the socket and clean up the network layer before disposing this object.
 */
ServerSocket::~ServerSocket()
{
	closesocket(listeningSocket);
#ifdef WIN32
	WSACleanup();
#endif
}

/**
 * initialize the windows socket layer and start a server socket on the specified port.
 */
BOOL ServerSocket::initSocket(short port)
{
#ifdef WIN32
	WSAData wsaData;
#endif
	int nRet;
	int one = 1;

	nPort = port;

#ifdef WIN32
	// Start Winsock up
	nRet = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (nRet != 0) {
		cerr << "WSAStartup() returned error code " << nRet << "." << endl;
		return FALSE;
	}
#endif

	listeningSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
	if (listeningSocket == INVALID_SOCKET) {
#if defined(WIN32)
		cerr << "Could not create listening socket: " << WSAGetLastError() << endl;
		WSACleanup();
#elif defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
		perror("Could not create listening socket:");
#endif
		return FALSE;
	}
#if defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
	setsockopt(listeningSocket,SOL_SOCKET,SO_REUSEADDR,&one,sizeof(one));
#endif
 
	SOCKADDR_IN sinInterface;
	sinInterface.sin_family = AF_INET;
	sinInterface.sin_addr.s_addr = INADDR_ANY;
	sinInterface.sin_port = htons(nPort);
#ifdef WIN32
	nRet = bind(listeningSocket, (LPSOCKADDR)&sinInterface, sizeof(sinInterface));
#elif defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
	nRet = bind(listeningSocket, (struct sockaddr*)&sinInterface, sizeof(sinInterface));
#endif
	if (nRet == SOCKET_ERROR) {
#ifdef WIN32
		cerr << "Bind error: " << WSAGetLastError() << endl;
		closesocket(listeningSocket);
		WSACleanup();
#elif defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
		perror("bind error:");
		closesocket(listeningSocket);
#endif
		return FALSE;
	}

	nRet = listen(listeningSocket, SOMAXCONN);
	if (nRet == SOCKET_ERROR) {
#ifdef WIN32
		cerr << "Listen error: " << WSAGetLastError() << endl;
		closesocket(listeningSocket);
		WSACleanup();
#elif defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
		perror("listen error:");
		closesocket(listeningSocket);
#endif
		return FALSE;
	}
	return TRUE;
} /* initSocket */

/**
 * this method will wait for a new incoming connection on the server port.
 */
ClientSocket* ServerSocket::acceptConnection()
{
	sockaddr_in sinRemote;
	int nAddrSize = sizeof(sinRemote);

	SOCKET client = accept(listeningSocket, (sockaddr*)&sinRemote, 
#if defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
		(unsigned int*)
#endif
		&nAddrSize);
	if (client == INVALID_SOCKET) {
#ifdef WIN32
		cerr << "Could not accept connection: " << WSAGetLastError() << endl;
#else
		perror("Could not accept connection: ");
#endif
		return NULL;
	}
#if defined(LINUX) || defined(linux) || defined(UNIX) || defined(unix)
        fcntl (client, F_SETFL, 0);	//sock?
#endif
	
	printf("Accepted connection from %s:%d.\n",inet_ntoa(sinRemote.sin_addr),
						   ntohs(sinRemote.sin_port));

	if (client == SOCKET_ERROR) {
		return NULL;
	}
	ClientSocket* cs = new ClientSocket(client);
	cs->host = inet_ntoa(sinRemote.sin_addr);
	cs->port = ntohs(sinRemote.sin_port);
	return cs;
} /* accept */

==> caiviar-0.3.1.tar.gz: caiviar-0.3.1/server/ServerSocket.cpp <==
/* ServerSocket.cpp

java-like implementation of sockets.

Part of the caiviar package. 

Copyright (c) 2002 MobileX AG, http://www.mobilexag.de
Copyright (c) 2002 Peter Dikant <peter.dikant@mobilexag.de>
Copyright (c) 2002 Matthias Kramm <kramm@quiss.org>

This file is distributed under the GPL, see file COPYING for details. */

#include "../config.h"
#if defined(LINUX) || defined(CYGWIN)
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <errno.h>
#include <time.h>
#include <memory.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <pthread.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <net/if.h>
#include <netdb.h>
#elif defined(WIN32)
#ifndef __windows_h_included__
#define __windows_h_included__
#include <windows.h>
#endif
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#endif
#include "ServerSocket.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#if defined(LINUX) || defined(CYGWIN)
static void Sleep(int useconds)
{
	usleep(useconds);
}
static void closesocket(int socket)
{
	close(socket);
}
static const char FALSE = 0;
static const char TRUE = 1;
typedef sockaddr_in SOCKADDR_IN;
typedef sockaddr_in* LPSOCKADDR;
#endif

ServerSocket::ServerSocket() 
{

}

/**
 * close the socket and clean up the network layer before disposing this object.
 */
ServerSocket::~ServerSocket()
{
	closesocket(listeningSocket);
#ifdef WIN32
	WSACleanup();
#endif
}

/**
 * initialize the windows socket layer and start a server socket on the specified port.
 */
int ServerSocket::initSocket(short port)
{
#ifdef WIN32
	WSAData wsaData;
#endif
	int nRet;
	int one = 1;

	nPort = port;

#ifdef WIN32
	// Start Winsock up
	nRet = WSAStartup(MAKEWORD(2, 2), &wsaData);
	if (nRet != 0) {
		cerr << "WSAStartup() returned error code " << nRet << "." << endl;
		return FALSE;
	}
#endif

	listeningSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
	if (listeningSocket == INVALID_SOCKET) {
#if defined(WIN32)
		cerr << "Could not create listening socket: " << WSAGetLastError() << endl;
		WSACleanup();
#elif defined(LINUX) || defined(CYGWIN)
		perror("Could not create listening socket:");
#endif
		return FALSE;
	}
#if defined(LINUX) || defined(CYGWIN)
	setsockopt(listeningSocket,SOL_SOCKET,SO_REUSEADDR,&one,sizeof(one));
#endif
 
	SOCKADDR_IN sinInterface;
	sinInterface.sin_family = AF_INET;
	sinInterface.sin_addr.s_addr = INADDR_ANY;
	sinInterface.sin_port = htons(nPort);
#ifdef WIN32
	nRet = bind(listeningSocket, (LPSOCKADDR)&sinInterface, sizeof(sinInterface));
#elif defined(LINUX) || defined(CYGWIN)
	nRet = bind(listeningSocket, (struct sockaddr*)&sinInterface, sizeof(sinInterface));
#endif
	if (nRet == SOCKET_ERROR) {
#ifdef WIN32
		cerr << "Bind error: " << WSAGetLastError() << endl;
		closesocket(listeningSocket);
		WSACleanup();
#elif defined(LINUX) || defined(CYGWIN)
		perror("bind error:");
		closesocket(listeningSocket);
#endif
		return FALSE;
	}

	nRet = listen(listeningSocket, SOMAXCONN);
	if (nRet == SOCKET_ERROR) {
#ifdef WIN32
		cerr << "Listen error: " << WSAGetLastError() << endl;
		closesocket(listeningSocket);
		WSACleanup();
#elif defined(LINUX) || defined(CYGWIN)
		perror("listen error:");
		closesocket(listeningSocket);
#endif
		return FALSE;
	}
	return TRUE;
} /* initSocket */

/**
 * this method will wait for a new incoming connection on the server port.
 */
ClientSocket* ServerSocket::acceptConnection()
{
	sockaddr_in sinRemote;
	int nAddrSize = sizeof(sinRemote);

	SOCKET client = accept(listeningSocket, (sockaddr*)&sinRemote, 
#if !defined(CYGWIN) && defined(LINUX)
		(unsigned int*)
#endif
		&nAddrSize);
	if (client == INVALID_SOCKET) {
#ifdef WIN32
		cerr << "Could not accept connection: " << WSAGetLastError() << endl;
#else
		perror("Could not accept connection: ");
#endif
		return NULL;
	}
#if defined(LINUX) || defined(CYGWIN)
        fcntl (client, F_SETFL, 0);	//sock?
#endif
	
	printf("Accepted connection from %s:%d.\n",inet_ntoa(sinRemote.sin_addr),
						   ntohs(sinRemote.sin_port));

	if (client == SOCKET_ERROR) {
		return NULL;
	}
	ClientSocket* cs = new ClientSocket(client);
	cs->host = inet_ntoa(sinRemote.sin_addr);
	cs->port = ntohs(sinRemote.sin_port);
	return cs;
} /* accept */
